@@ -8,8 +8,9 @@ mod rust_geo_python {
 
     use geo::orient::{Direction, Orient};
     use geo::{
-        Area, BooleanOps, Buffer, Contains, Distance, Euclidean, HausdorffDistance, LineString,
-        MultiLineString, MultiPoint, MultiPolygon, Point, Polygon, Simplify, unary_union,
+        Area, BooleanOps, Buffer, Contains, ContainsProperly, Distance, Euclidean,
+        HausdorffDistance, LineString, MultiLineString, MultiPoint, MultiPolygon, Point, Polygon,
+        Simplify, unary_union,
     };
     use ndarray::parallel::prelude::IntoParallelIterator;
     use ndarray::{ArrayView1, ArrayView2};
@@ -673,6 +674,170 @@ mod rust_geo_python {
             }
         }
 
+        fn contains(&self, rhs: &Shape) -> bool {
+            match (&self.inner, &rhs.inner) {
+                (Shapes::Point(p), Shapes::Point(q)) => p.as_ref().contains(q.as_ref()),
+                (Shapes::LineString(p), Shapes::Point(q)) => p.as_ref().contains(q.as_ref()),
+                (Shapes::Point(p), Shapes::LineString(q)) => p.as_ref().contains(q.as_ref()),
+                (Shapes::LineString(p), Shapes::LineString(q)) => p.as_ref().contains(q.as_ref()),
+                (Shapes::MultiLineString(p), Shapes::Point(q)) => p.as_ref().contains(q.as_ref()),
+                (Shapes::MultiLineString(p), Shapes::LineString(q)) => {
+                    p.as_ref().contains(q.as_ref())
+                }
+                (Shapes::MultiLineString(p), Shapes::MultiLineString(q)) => {
+                    p.as_ref().contains(q.as_ref())
+                }
+                (Shapes::Point(p), Shapes::MultiLineString(q)) => p.as_ref().contains(q.as_ref()),
+                (Shapes::LineString(p), Shapes::MultiLineString(q)) => {
+                    p.as_ref().contains(q.as_ref())
+                }
+                (Shapes::Polygon(p), Shapes::Point(q)) => p.as_ref().contains(q.as_ref()),
+                (Shapes::Polygon(p), Shapes::LineString(q)) => p.as_ref().contains(q.as_ref()),
+                (Shapes::Polygon(p), Shapes::MultiLineString(q)) => p.as_ref().contains(q.as_ref()),
+                (Shapes::Polygon(p), Shapes::Polygon(q)) => p.as_ref().contains(q.as_ref()),
+                (Shapes::Point(p), Shapes::Polygon(q)) => p.as_ref().contains(q.as_ref()),
+                (Shapes::LineString(p), Shapes::Polygon(q)) => p.as_ref().contains(q.as_ref()),
+                (Shapes::MultiLineString(p), Shapes::Polygon(q)) => p.as_ref().contains(q.as_ref()),
+                (Shapes::MultiPolygon(p), Shapes::Point(q)) => p.as_ref().contains(q.as_ref()),
+                (Shapes::MultiPolygon(p), Shapes::LineString(q)) => p.as_ref().contains(q.as_ref()),
+                (Shapes::MultiPolygon(p), Shapes::MultiLineString(q)) => {
+                    p.as_ref().contains(q.as_ref())
+                }
+                (Shapes::MultiPolygon(p), Shapes::Polygon(q)) => p.as_ref().contains(q.as_ref()),
+                (Shapes::MultiPolygon(p), Shapes::MultiPolygon(q)) => {
+                    p.as_ref().contains(q.as_ref())
+                }
+                (Shapes::Point(p), Shapes::MultiPolygon(q)) => p.as_ref().contains(q.as_ref()),
+                (Shapes::LineString(p), Shapes::MultiPolygon(q)) => p.as_ref().contains(q.as_ref()),
+                (Shapes::MultiLineString(p), Shapes::MultiPolygon(q)) => {
+                    p.as_ref().contains(q.as_ref())
+                }
+                (Shapes::Polygon(p), Shapes::MultiPolygon(q)) => p.as_ref().contains(q.as_ref()),
+                (Shapes::MultiPoint(p), Shapes::Point(q)) => p.as_ref().contains(q.as_ref()),
+                (Shapes::MultiPoint(p), Shapes::LineString(q)) => p.as_ref().contains(q.as_ref()),
+                (Shapes::MultiPoint(p), Shapes::MultiLineString(q)) => {
+                    p.as_ref().contains(q.as_ref())
+                }
+                (Shapes::MultiPoint(p), Shapes::Polygon(q)) => p.as_ref().contains(q.as_ref()),
+                (Shapes::MultiPoint(p), Shapes::MultiPolygon(q)) => p.as_ref().contains(q.as_ref()),
+                (Shapes::MultiPoint(p), Shapes::MultiPoint(q)) => p.as_ref().contains(q.as_ref()),
+                (Shapes::Point(p), Shapes::MultiPoint(q)) => p.as_ref().contains(q.as_ref()),
+                (Shapes::LineString(p), Shapes::MultiPoint(q)) => p.as_ref().contains(q.as_ref()),
+                (Shapes::MultiLineString(p), Shapes::MultiPoint(q)) => {
+                    p.as_ref().contains(q.as_ref())
+                }
+                (Shapes::Polygon(p), Shapes::MultiPoint(q)) => p.as_ref().contains(q.as_ref()),
+                (Shapes::MultiPolygon(p), Shapes::MultiPoint(q)) => p.as_ref().contains(q.as_ref()),
+            }
+        }
+
+        fn contains_properly(&self, rhs: &Shape) -> bool {
+            match (&self.inner, &rhs.inner) {
+                (Shapes::Point(p), Shapes::Point(q)) => p.as_ref().contains_properly(q.as_ref()),
+                (Shapes::LineString(p), Shapes::Point(q)) => {
+                    p.as_ref().contains_properly(q.as_ref())
+                }
+                (Shapes::Point(p), Shapes::LineString(q)) => {
+                    p.as_ref().contains_properly(q.as_ref())
+                }
+                (Shapes::LineString(p), Shapes::LineString(q)) => {
+                    p.as_ref().contains_properly(q.as_ref())
+                }
+                (Shapes::MultiLineString(p), Shapes::Point(q)) => {
+                    p.as_ref().contains_properly(q.as_ref())
+                }
+                (Shapes::MultiLineString(p), Shapes::LineString(q)) => {
+                    p.as_ref().contains_properly(q.as_ref())
+                }
+                (Shapes::MultiLineString(p), Shapes::MultiLineString(q)) => {
+                    p.as_ref().contains_properly(q.as_ref())
+                }
+                (Shapes::Point(p), Shapes::MultiLineString(q)) => {
+                    p.as_ref().contains_properly(q.as_ref())
+                }
+                (Shapes::LineString(p), Shapes::MultiLineString(q)) => {
+                    p.as_ref().contains_properly(q.as_ref())
+                }
+                (Shapes::Polygon(p), Shapes::Point(q)) => p.as_ref().contains_properly(q.as_ref()),
+                (Shapes::Polygon(p), Shapes::LineString(q)) => {
+                    p.as_ref().contains_properly(q.as_ref())
+                }
+                (Shapes::Polygon(p), Shapes::MultiLineString(q)) => {
+                    p.as_ref().contains_properly(q.as_ref())
+                }
+                (Shapes::Polygon(p), Shapes::Polygon(q)) => {
+                    p.as_ref().contains_properly(q.as_ref())
+                }
+                (Shapes::Point(p), Shapes::Polygon(q)) => p.as_ref().contains_properly(q.as_ref()),
+                (Shapes::LineString(p), Shapes::Polygon(q)) => {
+                    p.as_ref().contains_properly(q.as_ref())
+                }
+                (Shapes::MultiLineString(p), Shapes::Polygon(q)) => {
+                    p.as_ref().contains_properly(q.as_ref())
+                }
+                (Shapes::MultiPolygon(p), Shapes::Point(q)) => {
+                    p.as_ref().contains_properly(q.as_ref())
+                }
+                (Shapes::MultiPolygon(p), Shapes::LineString(q)) => {
+                    p.as_ref().contains_properly(q.as_ref())
+                }
+                (Shapes::MultiPolygon(p), Shapes::MultiLineString(q)) => {
+                    p.as_ref().contains_properly(q.as_ref())
+                }
+                (Shapes::MultiPolygon(p), Shapes::Polygon(q)) => {
+                    p.as_ref().contains_properly(q.as_ref())
+                }
+                (Shapes::MultiPolygon(p), Shapes::MultiPolygon(q)) => {
+                    p.as_ref().contains_properly(q.as_ref())
+                }
+                (Shapes::Point(p), Shapes::MultiPolygon(q)) => {
+                    p.as_ref().contains_properly(q.as_ref())
+                }
+                (Shapes::LineString(p), Shapes::MultiPolygon(q)) => {
+                    p.as_ref().contains_properly(q.as_ref())
+                }
+                (Shapes::MultiLineString(p), Shapes::MultiPolygon(q)) => {
+                    p.as_ref().contains_properly(q.as_ref())
+                }
+                (Shapes::Polygon(p), Shapes::MultiPolygon(q)) => {
+                    p.as_ref().contains_properly(q.as_ref())
+                }
+                (Shapes::MultiPoint(p), Shapes::Point(q)) => {
+                    p.as_ref().contains_properly(q.as_ref())
+                }
+                (Shapes::MultiPoint(p), Shapes::LineString(q)) => {
+                    p.as_ref().contains_properly(q.as_ref())
+                }
+                (Shapes::MultiPoint(p), Shapes::MultiLineString(q)) => {
+                    p.as_ref().contains_properly(q.as_ref())
+                }
+                (Shapes::MultiPoint(p), Shapes::Polygon(q)) => {
+                    p.as_ref().contains_properly(q.as_ref())
+                }
+                (Shapes::MultiPoint(p), Shapes::MultiPolygon(q)) => {
+                    p.as_ref().contains_properly(q.as_ref())
+                }
+                (Shapes::MultiPoint(p), Shapes::MultiPoint(q)) => {
+                    p.as_ref().contains_properly(q.as_ref())
+                }
+                (Shapes::Point(p), Shapes::MultiPoint(q)) => {
+                    p.as_ref().contains_properly(q.as_ref())
+                }
+                (Shapes::LineString(p), Shapes::MultiPoint(q)) => {
+                    p.as_ref().contains_properly(q.as_ref())
+                }
+                (Shapes::MultiLineString(p), Shapes::MultiPoint(q)) => {
+                    p.as_ref().contains_properly(q.as_ref())
+                }
+                (Shapes::Polygon(p), Shapes::MultiPoint(q)) => {
+                    p.as_ref().contains_properly(q.as_ref())
+                }
+                (Shapes::MultiPolygon(p), Shapes::MultiPoint(q)) => {
+                    p.as_ref().contains_properly(q.as_ref())
+                }
+            }
+        }
+
         fn to_wkt(&self) -> String {
             match &self.inner {
                 Shapes::Point(p) => p.as_ref().wkt_string(),
